@@ -1,9 +1,9 @@
-# Copyright (C) 2019-2020 Ruhr West University of Applied Sciences, Bottrop, Germany
-# AND Visteon Electronics Germany GmbH, Kerpen, Germany
+# Copyright (C) 2019-2021 Ruhr West University of Applied Sciences, Bottrop, Germany
+# AND Elektronische Fahrwerksysteme GmbH, Gaimersheim Germany
 #
-# This Source Code Form is subject to the terms of the Mozilla Public
-# License, v. 2.0. If a copy of the MPL was not distributed with this
-# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+# This Source Code Form is subject to the terms of the Apache License 2.0
+# If a copy of the APL2 was not distributed with this
+# file, You can obtain one at https://www.apache.org/licenses/LICENSE-2.0.txt.
 
 """
 Methods for measuring miscalibration. The common methods are given with the
@@ -22,9 +22,13 @@ Available classes
    ACE
    ECE
    MCE
+   MMCE
+   PICP
 """
 
 from .ACE import ACE
 from .ECE import ECE
 from .MCE import MCE
-from .Miscalibration import _Miscalibration
\ No newline at end of file
+from .Miscalibration import _Miscalibration
+from .PICP import PICP
+from .MMCE import MMCE
