@@ -1,9 +1,9 @@
-# Copyright (C) 2019-2020 Ruhr West University of Applied Sciences, Bottrop, Germany
-# AND Visteon Electronics Germany GmbH, Kerpen, Germany
+# Copyright (C) 2019-2021 Ruhr West University of Applied Sciences, Bottrop, Germany
+# AND Elektronische Fahrwerksysteme GmbH, Gaimersheim Germany
 #
-# This Source Code Form is subject to the terms of the Mozilla Public
-# License, v. 2.0. If a copy of the MPL was not distributed with this
-# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+# This Source Code Form is subject to the terms of the Apache License 2.0
+# If a copy of the APL2 was not distributed with this
+# file, You can obtain one at https://www.apache.org/licenses/LICENSE-2.0.txt.
 
 """
 Regularization methods which are applied during model training. These methods should achieve a
@@ -18,7 +18,13 @@ Available functions
    :toctree: _autosummary_regularization_func
 
    confidence_penalty
+   ConfidencePenalty
+   MMCEPenalty
+   DCAPenalty
 """
 
 
 from .ConfidencePenalty import confidence_penalty
+from .ConfidencePenalty import ConfidencePenalty
+from .MMCEPenalty import MMCEPenalty
+from .DCAPenalty import DCAPenalty
