@@ -1,9 +1,9 @@
-# Copyright (C) 2019-2020 Ruhr West University of Applied Sciences, Bottrop, Germany
-# AND Visteon Electronics Germany GmbH, Kerpen, Germany
+# Copyright (C) 2019-2021 Ruhr West University of Applied Sciences, Bottrop, Germany
+# AND Elektronische Fahrwerksysteme GmbH, Gaimersheim Germany
 #
-# This Source Code Form is subject to the terms of the Mozilla Public
-# License, v. 2.0. If a copy of the MPL was not distributed with this
-# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+# This Source Code Form is subject to the terms of the Apache License 2.0
+# If a copy of the APL2 was not distributed with this
+# file, You can obtain one at https://www.apache.org/licenses/LICENSE-2.0.txt.
 
 """
 Scaling methods for confidence calibration.
@@ -17,6 +17,7 @@ Available classes
    :toctree: _autosummary_scaling
    :template: custom_class.rst
 
+   AbstractLogisticRegression
    LogisticCalibration
    LogisticCalibrationDependent
    TemperatureScaling
@@ -24,6 +25,7 @@ Available classes
    BetaCalibrationDependent
 """
 
+from .AbstractLogisticRegression import AbstractLogisticRegression
 
 from .LogisticCalibration import LogisticCalibration
 from .LogisticCalibrationDependent import LogisticCalibrationDependent
